package com.jboss.sample;

import Sender;

import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.*;

/**
 * @author Sebastian Hennebrueder
 */
public class HelloWorldTest {
	private static EntityManagerFactory emf;

	@BeforeClass
	public static void beforeTest(){
		emf = Persistence.createEntityManagerFactory("samplePU");
	}

	/**
	 * TODO 1) Complete the configuration
	 * The Hibernate configuration is incorrect. Update the dialect
	 * in the <pre>persistence.xml</pre> for the HSQL database.
	 *
	 * This test should run fine, if the configuration has been
	 * fixed.
	 */
	@Test
	public void checkConfiguration(){
		EntityManager em = emf.createEntityManager();
		em.getTransaction().begin();

		final Calendar birthday = Calendar.getInstance();
		birthday.add(Calendar.YEAR, -25);
		Sender sender = new Sender("Jane", birthday.getTime(), Sex.FEMALE);
		em.persist(sender);

		em.getTransaction().commit();
		em.close();
	}

	/**
	 * TODO 2) Complete the sender mapping
	 * The properties birthday and sex are not persisted. Complete the mapping of
	 * the {@link Sender}
	 */
	@Test
	public void persistSender(){
		// create a sender
		EntityManager em = emf.createEntityManager();
		em.getTransaction().begin();

		final Calendar birthday = Calendar.getInstance();
		birthday.set(1980, 3, 22);
		Sender sender = new Sender("Peter", birthday.getTime(), Sex.MALE);
		em.persist(sender);

		em.getTransaction().commit();
		em.close();

		// validate that everything was saved
		em = emf.createEntityManager();
		em.getTransaction().begin();

		Sender reloaded = em.find(Sender.class, sender.getId());

		em.getTransaction().commit();
		em.close();
		final GregorianCalendar reloadedDate = new GregorianCalendar();
		reloadedDate.setTime(reloaded.getBirthday());

		Assert.assertEquals("Peter", reloaded.getName());
		Assert.assertEquals(birthday.get(Calendar.YEAR), reloadedDate.get(Calendar.YEAR));
		Assert.assertEquals(birthday.get(Calendar.MONTH), reloadedDate.get(Calendar.MONTH));
		Assert.assertEquals(birthday.get(Calendar.DATE), reloadedDate.get(Calendar.DATE));
		Assert.assertEquals(Sex.MALE, reloaded.getSex());
	}

	/**
	 * TODO 3) Complete the sender mapping
	 * The property <pre>address</pre> is marked as {@code @Transient}. It is not persisted. Map the address as component.
	 * Complete the mapping in {@link Address}
	 */
	@Test
	public void persistSenderAddress(){
		// create a sender with an address
		EntityManager em = emf.createEntityManager();
		em.getTransaction().begin();

		final String street = "Old Street";
		final String city = "Bad Vilbel";
		final String country = "Germany";

		Address address = new Address(street, city, country);
		Sender sender = new Sender("Holger", address);
		em.persist(sender);

		em.getTransaction().commit();
		em.close();

		// validate that everything was saved
		em = emf.createEntityManager();
		em.getTransaction().begin();

		Sender reloaded = em.find(Sender.class, sender.getId());

		em.getTransaction().commit();
		em.close();

		Assert.assertEquals("Holger", reloaded.getName());
		Assert.assertNotNull(reloaded.getAddress());
		Assert.assertEquals(street, reloaded.getAddress().getStreet());
		Assert.assertEquals(city, reloaded.getAddress().getCity());
		Assert.assertEquals(country, reloaded.getAddress().getCountry());
	}

	/**
	 * TODO 4) Create a relation between {@link Message} and {@link Sender}
	 * The message has a reference to the sender. Add the annotations in the class {@link Message}
	 * and make sure that the relation is created and saved.
	 * Complete the code in this method.
	 */
	@Test
	public void createRelation(){
		 // create a message with a sender
		EntityManager em = emf.createEntityManager();
		em.getTransaction().begin();

		final String georg = "Georg";
		Sender sender = new Sender(georg);
		Message message = new Message("Hello world", sender);
		em.persist(message);

		em.getTransaction().commit();
		em.close();

		// validate that the relation was saved
		em = emf.createEntityManager();
		em.getTransaction().begin();

		Message reloaded = em.find(Message.class, message.getId());

		em.getTransaction().commit();
		em.close();

		Assert.assertNotNull(reloaded.getSender());
		Assert.assertEquals(georg, reloaded.getSender().getName());
	}

	/**
	 * TODO 5) Update the sender
	 * The name of the sender was changed. Save the change in the database using the JPA API. Then the sex was changed as well.
	 * Save the sender again. Validate that the expected name and sex is stored in the database.
	 * Complete the code in this method
	 */
	@Test
	public void updateSender(){
		// create a sender with an address
		Sender sender = new Sender("Andrea", new Date(), Sex.MALE);

		EntityManager em = emf.createEntityManager();
		em.getTransaction().begin();
		em.persist(sender);
		em.getTransaction().commit();
		em.close();

		// The name is changed here
		final String davorka = "Davorka";
		sender.setName(davorka);

		// save the changes
		em = emf.createEntityManager();
		em.getTransaction().begin();
		//sender=
		em.merge(sender);
		em.getTransaction().commit();
		em.close();

		// validate that the name was saved
		em = emf.createEntityManager();
		em.getTransaction().begin();
		Sender reloaded = em.find(Sender.class, sender.getId());
		em.getTransaction().commit();
		em.close();

		Assert.assertEquals(davorka, reloaded.getName());

		// The sex is corrected
		sender.setSex(Sex.FEMALE);

		// save the changes here
		em = emf.createEntityManager();
		em.getTransaction().begin();
		/*sender=*/em.merge(sender);
		em.getTransaction().commit();
		em.close();

		// validate that everything was saved
		em = emf.createEntityManager();
		em.getTransaction().begin();
		reloaded = em.find(Sender.class, sender.getId());
		em.getTransaction().commit();
		em.close();

		Assert.assertEquals(davorka, reloaded.getName());
		Assert.assertEquals(Sex.FEMALE, reloaded.getSex());

	}

/**
	 * TODO 6) Query senders
	 * The name of the sender was changed. Save the change in the database using the JPA API. Then the sex was changed as well.
	 * Save the sender again. Validate that the expected name and sex is stored in the database.
	 * Complete the code in this method
	 */
	@Test
	public void querySender(){

		EntityManager em = emf.createEntityManager();
		em.getTransaction().begin();

		System.out.println("Query all sender and print them");
		// add the first query here
		List<Sender> resultList = em.createQuery("select s from Sender s", Sender.class).getResultList();
		printResult(resultList);

		em.clear();

		System.out.println("Query all sender named Holger");
		// add the second query here
		//resultList = Collections.emptyList();
		resultList = em.createQuery("select s from Sender s where s.name = :name", Sender.class)
		.setParameter("name", "Holger").getResultList();

		printResult(resultList);


		em.getTransaction().commit();
		em.close();
	}

	private void printResult(List<Sender> resultList) {
		for (Sender sender : resultList) {
			System.out.println("  "+sender);
		}
	}


}
