package auction.service;

import auction.model.Item;
import auction.model.Users;
import auction.test.DatabaseSetup;
import org.dbunit.operation.DatabaseOperation;
import org.jboss.weld.environment.se.Weld;
import org.jboss.weld.environment.se.WeldContainer;
import org.jboss.weld.environment.se.events.ContainerInitialized;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import static org.junit.Assert.assertEquals;

/**
 * @author Sebastian Hennebrueder
 */
public class MappingTest {

	final Logger logger = LoggerFactory.getLogger(MappingTest.class);

	private static WeldContainer weldContainer;

	@BeforeClass
	public static void setup() {

		// start the container first to let the Hibernate generate the tables
		weldContainer = new Weld().initialize();
		weldContainer.event().select(ContainerInitialized.class).fire(new ContainerInitialized());


		DatabaseSetup setup = new DatabaseSetup();
		setup.setDataFile("auction/service/basedata.xml").setup();

	}

	@Test
	public void rampUpTest() {
		final UserService userService = weldContainer.instance().select(UserService.class).get();
		final AuctionService auctionService = weldContainer.instance().select(AuctionService.class).get();
		// Prepare a user object

		Users user = userService.findByIdInitializeItems(1l, false);


		// Make a new auction item persistent
		Calendar startDate = GregorianCalendar.getInstance();
		Calendar endDate = GregorianCalendar.getInstance();
		endDate.add(Calendar.DAY_OF_YEAR, 3);


		Item newItem =
				new Item();
		newItem.setCreated(new Date());
		newItem.setDescription("Test Description");
		newItem.setEndDate(endDate.getTime());
		newItem.setInitialPrice(new BigDecimal(5L));
		newItem.setInitialPriceCurrency("USD");
		newItem.setReservePrice(new BigDecimal(5L));
		newItem.setReservePriceCurrency("USD");
		newItem.setItemName( "Testitem");
		newItem.setStartDate(startDate.getTime());
		newItem.setItemState("PENDING");
		newItem.setUsersBySellerId(user);

		auctionService.create(newItem);


		// Direct SQL query for database state in auto-commit mode
		final EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
				EntityManagerFactory.class).get();

		final EntityManager em = entityManagerFactory.createEntityManager();
		em.getTransaction().begin();
		logger.debug("id is: "+newItem.getItemId());
		Object[] result = (Object[])
				em.createNativeQuery(
						"select INITIAL_PRICE as IP," +
								"       SELLER_ID as SID from ITEM where ITEM_ID = :itemId")
						.setParameter("itemId", newItem.getItemId())
						.getSingleResult();
		em.getTransaction().commit();
		em.close();


		// Assert correctness of state
		assertEquals(BigDecimal.class, result[0].getClass());
		assertEquals(result[0], newItem.getInitialPrice());



	}


}
