package auction.service;

import auction.model.User;

import java.util.List;

/**
 * @author Sebastian Hennebrueder
 */
public interface UserService {
	/**
	 * Finds a user by a given name
	 * @param name - a username
	 * @return Either {@code null} or the found user
	 */
	User findByName(String name);

	/**
	 * finds a user by the given id
	 * @param id - the id of the user
	 * @param lock - a flag indicating if the user should be locked
	 * @return the found user or {@code null} if none was found
	 */
	User findById(Long id, boolean lock);

	/**
	 * finds a user by the given id and initialize the itemsForSale collection
	 * @param id - the id of the user
	 * @param lock - a flag indicating if the user should be locked
	 * @return the found user or {@code null} if none was found
	 */
	User findByIdInitializeItems(long id, boolean lock);

	/**
	 * Finds all users
	 *
	 * @return all found users or an empty list if none was found
	 */
	List<User> findAll();

	/**
	 * Saves a user in the database
	 * @param user - the user to save
	 */
	void save(User user);
}
