
create table ADDRESS (ADDRESS_ID bigint not null, CITY varchar(255) not null, STREET varchar(255) not null, OBJ_VERSION integer, ZIPCODE varchar(16) not null, primary key (ADDRESS_ID))
create table BID (BID_ID bigint generated by default as identity (start with 1), BID_AMOUNT numeric, BID_AMOUNT_CURRENCY varchar(3), CREATED timestamp not null, BIDDER_ID bigint not null, ITEM_ID bigint not null, BID_POSITION integer, primary key (BID_ID))
create table BILLING_ADDRESS (CITY varchar(255), STREET varchar(255), ZIPCODE varchar(16), USER_ID bigint not null, primary key (USER_ID))
create table BILLING_DETAILS (BILLING_DETAILS_TYPE varchar(31) not null, BILLING_DETAILS_ID bigint generated by default as identity (start with 1), CREATED timestamp not null, OWNER varchar(255) not null, OBJ_VERSION integer, BA_ACCOUNT varchar(16), BA_BANKNAME varchar(255), BA_SWIFT varchar(15), USER_ID bigint, primary key (BILLING_DETAILS_ID))
create table CATEGORIZED_ITEM (CATEGORY_ID bigint not null, ITEM_ID bigint not null, ADDED_ON timestamp, ADDED_BY_USER varchar(16) not null, primary key (CATEGORY_ID, ITEM_ID))
create table CATEGORIZED_ITEM_COMPONENTS (CATEGORY_ID bigint not null, ADDED_ON timestamp not null, ITEM_ID bigint not null, ADDED_BY_USER varchar(16) not null, primary key (CATEGORY_ID, ADDED_ON, ITEM_ID, ADDED_BY_USER))
create table CATEGORY (CATEGORY_ID bigint generated by default as identity (start with 1), CREATED timestamp not null, CATEGORY_NAME varchar(255) not null, OBJ_VERSION integer, PARENT_CATEGORY_ID bigint, primary key (CATEGORY_ID), unique (CATEGORY_NAME, PARENT_CATEGORY_ID))
create table CATEGORY_ITEM (CATEGORY_ID bigint not null, ITEM_ID bigint not null, DISPLAY_POSITION integer not null, primary key (CATEGORY_ID, DISPLAY_POSITION))
create table CATEGORY_ITEMS_BY_USER (CATEGORY_ID bigint not null, USER_ID bigint not null, ITEM_ID bigint not null, primary key (CATEGORY_ID, ITEM_ID))
create table COMMENT (COMMENT_ID bigint generated by default as identity (start with 1), CREATED timestamp not null, RATING varchar(255) not null, COMMENT_TEXT varchar(4000), OBJ_VERSION integer, FROM_USER_ID bigint not null, ABOUT_ITEM_ID bigint not null, primary key (COMMENT_ID))
create table CREDIT_CARD (CC_EXP_MONTH varchar(2) not null, CC_EXP_YEAR varchar(4) not null, CC_NUMBER varchar(16) not null, CC_TYPE varchar(255) not null, CREDIT_CARD_ID bigint not null, primary key (CREDIT_CARD_ID))
create table ITEM (ITEM_ID bigint generated by default as identity (start with 1), APPROVAL_DATETIME timestamp, CREATED timestamp not null, DESCRIPTION varchar(4000) not null, END_DATE timestamp not null, INITIAL_PRICE numeric not null, INITIAL_PRICE_CURRENCY varchar(3) not null, ITEM_NAME varchar(255) not null, RESERVE_PRICE numeric not null, RESERVE_PRICE_CURRENCY varchar(3) not null, START_DATE timestamp not null, ITEM_STATE varchar(255) not null, OBJ_VERSION integer, APPROVED_BY_USER_ID bigint, SELLER_ID bigint not null, SUCCESSFUL_BID_ID bigint, primary key (ITEM_ID), check (START_DATE < END_DATE))
create table ITEM_BUYER (USER_ID bigint, ITEM_ID bigint not null, primary key (ITEM_ID))
create table ITEM_IMAGES (ITEM_ID bigint not null, FILENAME varchar(255), ITEM_IMAGE_ID bigint generated by default as identity (start with 1), primary key (ITEM_IMAGE_ID))
create table ITEM_SHIPMENT (ITEM_ID bigint, SHIPMENT_ID bigint not null, primary key (SHIPMENT_ID))
create table SHIPMENT (SHIPMENT_ID bigint generated by default as identity (start with 1), CREATED timestamp not null, INSPECTION_PERIOD_DAYS integer not null, SHIPMENT_STATE varchar(255) not null, OBJ_VERSION integer, BUYER_ID bigint not null, DELIVERY_ADDRESS_ID bigint not null, SELLER_ID bigint not null, primary key (SHIPMENT_ID))
create table USERS (USER_ID bigint generated by default as identity (start with 1), IS_ADMIN bit not null, CREATED timestamp not null, EMAIL varchar(255) not null, FIRSTNAME varchar(255) not null, HOME_CITY varchar(255), HOME_STREET varchar(255), HOME_ZIPCODE varchar(16), LASTNAME varchar(255) not null, "PASSWORD" varchar(12) not null, RANK integer not null, USERNAME varchar(16) not null, OBJ_VERSION integer, DEFAULT_BILLING_DETAILS_ID bigint, primary key (USER_ID), unique (USERNAME))
alter table ADDRESS add constraint FKE66327D43499D961 foreign key (ADDRESS_ID) references USERS
alter table BID add constraint FK_BIDDER_ID foreign key (BIDDER_ID) references USERS
alter table BID add constraint FK_ITEM_ID foreign key (ITEM_ID) references ITEM
alter table BILLING_ADDRESS add constraint FK1FF7F1F0DBE5E62A foreign key (USER_ID) references USERS
alter table BILLING_DETAILS add constraint FK_USER_ID foreign key (USER_ID) references USERS
alter table CATEGORIZED_ITEM add constraint FK_CATEGORIZED_ITEM_ITEM_ID foreign key (ITEM_ID) references ITEM
alter table CATEGORIZED_ITEM add constraint FK_CATEGORIZED_ITEM_CATEGORY_ID foreign key (CATEGORY_ID) references CATEGORY
alter table CATEGORIZED_ITEM_COMPONENTS add constraint FK_CATEGORIZED_ITEM_COMPONENT_ITEM_ID foreign key (ITEM_ID) references ITEM
alter table CATEGORIZED_ITEM_COMPONENTS add constraint FK_CATEGORIZED_ITEM_COMPONENT_CATEGORY_ID foreign key (CATEGORY_ID) references CATEGORY
alter table CATEGORY add constraint FK_CATEGORY_PARENT_ID foreign key (PARENT_CATEGORY_ID) references CATEGORY
alter table CATEGORY_ITEM add constraint FK_CATEGORY_ITEM_ITEM_ID foreign key (ITEM_ID) references ITEM
alter table CATEGORY_ITEM add constraint FK_CATEGORY_ITEM_CATEGORY_ID foreign key (CATEGORY_ID) references CATEGORY
alter table CATEGORY_ITEMS_BY_USER add constraint FK94CC139362C8412A foreign key (ITEM_ID) references ITEM
alter table CATEGORY_ITEMS_BY_USER add constraint FK_CATEGORY_ITEMS_BY_USER_CATEGORY_ID foreign key (CATEGORY_ID) references CATEGORY
alter table CATEGORY_ITEMS_BY_USER add constraint FK_CATEGORY_ITEMS_BY_USER_USER_ID foreign key (USER_ID) references USERS
alter table COMMENT add constraint FK_ABOUT_ITEM_ID foreign key (ABOUT_ITEM_ID) references ITEM
alter table COMMENT add constraint FK_FROM_USER_ID foreign key (FROM_USER_ID) references USERS
alter table CREDIT_CARD add constraint FK6FFB0096BEC2AB5D foreign key (CREDIT_CARD_ID) references BILLING_DETAILS
create index IDX_END_DATE on ITEM (END_DATE)
create index IDX_INITIAL_PRICE on ITEM (INITIAL_PRICE, INITIAL_PRICE_CURRENCY)
alter table ITEM add constraint FK_SUCCESSFUL_BID_ID foreign key (SUCCESSFUL_BID_ID) references BID
alter table ITEM add constraint FK_APPROVED_BY_USER_ID foreign key (APPROVED_BY_USER_ID) references USERS
alter table ITEM add constraint FK_SELLER_ID foreign key (SELLER_ID) references USERS
alter table ITEM_BUYER add constraint FKDE3D056762C8412A foreign key (ITEM_ID) references ITEM
alter table ITEM_BUYER add constraint FK_ITEM_BUYER_USER_ID foreign key (USER_ID) references USERS
alter table ITEM_IMAGES add constraint FK_ITEM_IMAGE_ITEM_ID foreign key (ITEM_ID) references ITEM
alter table ITEM_SHIPMENT add constraint FK3AA6E546B4FB37CA foreign key (SHIPMENT_ID) references SHIPMENT
alter table ITEM_SHIPMENT add constraint FK_ITEM_SHIPMENT_ITEM_ID foreign key (ITEM_ID) references ITEM
alter table SHIPMENT add constraint FK_SHIPMENT_BUYER_ID foreign key (BUYER_ID) references USERS
alter table SHIPMENT add constraint FK_SHIPMENT_SELLER_ID foreign key (SELLER_ID) references USERS
alter table SHIPMENT add constraint FK_SHIPMENT_DELIVERY_ADDRESS_ID foreign key (DELIVERY_ADDRESS_ID) references ADDRESS
alter table USERS add constraint FK_DEFAULT_BILLING_DETAILS_ID foreign key (DEFAULT_BILLING_DETAILS_ID) references BILLING_DETAILS
