package auction.service;

import auction.model.*;
import auction.test.DatabaseSetup;
import org.dbunit.operation.DatabaseOperation;
import org.hibernate.cfg.AnnotationConfiguration;
import org.hibernate.ejb.Ejb3Configuration;
import org.hibernate.tool.hbm2ddl.SchemaExport;
import org.jboss.weld.environment.se.Weld;
import org.jboss.weld.environment.se.WeldContainer;
import org.jboss.weld.environment.se.events.ContainerInitialized;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.*;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

/**
 * @author Sebastian Hennebrueder
 */
public class MappingTest {

    private static final String OLD_STREET = "Old street";
    private static final String HIGHWAY_3 = "Highway 3";
    private static final String BAD_VIBEL = "Bad Vibel";
    private static final String SAN_FRANCISCO = "San Francisco";

    final Logger logger = LoggerFactory.getLogger(MappingTest.class);

    private static WeldContainer weldContainer;

    private static Date today;
    private static Date nextWeek;

    @BeforeClass
    public static void setup() {
        final Calendar calendar = Calendar.getInstance();
        today = calendar.getTime();
        calendar.add(Calendar.DATE, 7);
        nextWeek = calendar.getTime();

        // start the container first to let the Hibernate generate the tables
        weldContainer = new Weld().initialize();
        weldContainer.event().select(ContainerInitialized.class).fire(new ContainerInitialized());


        DatabaseSetup setup = new DatabaseSetup();
        setup.setDataFile("auction/service/basedata.xml").setup();

    }


    @Test
    public void persistData() {

        User seller = new User("Sean", "Maier", "sean", "p", "sean@example.com");

        Item item = new Item("Bike", "A red bike", seller, today, nextWeek);

        Category bikeCategory = new Category("Bikes");
        Category treckingBikeCategory = new Category("Trecking bikes");
        bikeCategory.addChildCategory(treckingBikeCategory);

        treckingBikeCategory.addItem(item);
        EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
                EntityManagerFactory.class).get();

        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(seller);
        em.persist(bikeCategory);
        em.getTransaction().commit();
        em.close();

        Assert.assertNotNull(seller.getId());
        Assert.assertNotNull(bikeCategory.getId());
        Assert.assertNotNull(treckingBikeCategory.getId());
        Assert.assertNotNull(item.getId());

    }

    @Test
    public void removeComment() {
        User seller = new User("Karl", "Jones", "karl", "p", "karl@example.com");

        Item item = new Item("Bike", "A red bike", seller, today, nextWeek);
        Comment comment = new Comment(Rating.OK, "A really nice bike");
        item.addComment(comment);
        EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
                EntityManagerFactory.class).get();
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(seller);
        em.persist(item);
        em.getTransaction().commit();
        em.close();

        // we remove the comment now
        em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        Item merged = em.merge(item);
        merged.removeComment(comment);
        em.getTransaction().commit();
        em.close();

        em = entityManagerFactory.createEntityManager();
        Comment reloaded = em.find(Comment.class, comment.getId());
        Assert.assertNull(reloaded);


    }


}
