package auction.test;

import org.dbunit.IDatabaseTester;
import org.dbunit.JdbcDatabaseTester;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.dataset.ReplacementDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSetBuilder;
import org.dbunit.ext.hsqldb.HsqldbDataTypeFactory;
import org.dbunit.operation.DatabaseOperation;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Sebastian Hennebrueder
 */
public class DatabaseSetup {

	private CustomIDatabaseTester iDatabaseTester;

	private String classPathResource;

	public DatabaseSetup setDataFile(String classPathResource) {
		this.classPathResource = classPathResource;
		return this;
	}

	public void setup() {
		try {
			iDatabaseTester = new CustomIDatabaseTester("org.hsqldb.jdbcDriver",
					"jdbc:hsqldb:hsql://localhost", "sa", "");


			InputStream input =
					Thread.currentThread().getContextClassLoader()
							.getResourceAsStream(classPathResource);

			final FlatXmlDataSet flatXmlDataSet = new FlatXmlDataSetBuilder().build(input);
			ReplacementDataSet dataSet = new ReplacementDataSet(flatXmlDataSet);
			dataSet.addReplacementObject("[NULL]", null);

			iDatabaseTester.setDataSet(dataSet);

			iDatabaseTester.disableReferentialIntegrity();
			iDatabaseTester.onSetup();
			iDatabaseTester.enableReferentialIntegrity();

		} catch (Exception e) {
			throw new RuntimeException("Test setup failed " + e.getMessage(), e);
		}

	}


	/**
	 * DB Unit is missing a type for HSQL database
	 */
	public class CustomIDatabaseTester extends JdbcDatabaseTester {

		public CustomIDatabaseTester(String driverClass, String connectionUrl, String username,
																 String password) throws ClassNotFoundException {
			super(driverClass, connectionUrl, username, password);
		}

		@Override
		public IDatabaseConnection getConnection() throws Exception {
			IDatabaseConnection connection = super.getConnection();
			connection.getConfig().setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY,
								new HsqldbDataTypeFactory());

			return connection;
		}

		public void enableReferentialIntegrity(){
			try {
				super.getConnection().getConnection().prepareStatement("set referential_integrity TRUE")
												.execute();
			} catch (Exception e) {
				throw new RuntimeException(e.getMessage(), e);
			}

		}

		public void disableReferentialIntegrity(){
			try {
				super.getConnection().getConnection().prepareStatement("set referential_integrity FALSE")
												.execute();
			} catch (Exception e) {
				throw new RuntimeException(e.getMessage(), e);
			}
		}

	}
}
