package auction.service;

import auction.model.*;
import auction.test.DatabaseSetup;
import org.jboss.weld.environment.se.Weld;
import org.jboss.weld.environment.se.WeldContainer;
import org.jboss.weld.environment.se.events.ContainerInitialized;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.Root;
import java.util.*;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

/**
 * @author Sebastian Hennebrueder
 */
public class MappingTest {

    final Logger logger = LoggerFactory.getLogger(MappingTest.class);

    private static WeldContainer weldContainer;

    private static Date today;
    private static Date nextWeek;

    @BeforeClass
    public static void setup() {
        final Calendar calendar = Calendar.getInstance();
        today = calendar.getTime();
        calendar.add(Calendar.DATE, 7);
        nextWeek = calendar.getTime();

        // start the container first to let the Hibernate generate the tables
        weldContainer = new Weld().initialize();
        weldContainer.event().select(ContainerInitialized.class).fire(new ContainerInitialized());


        DatabaseSetup setup = new DatabaseSetup();
        setup.setDataFile("auction/service/basedata.xml").setup();

    }


    // TODO 2 - select users which name starts with jo
    @Test
    public void selectUser() {

        EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
                EntityManagerFactory.class).get();

        EntityManager em = entityManagerFactory.createEntityManager();
        List<User> users = null;
        em.close();

        Assert.assertEquals(3, users.size());
        for (User user : users) {
            Assert.assertTrue(user.getUsername().startsWith("jo"));
        }
    }

    // TODO 3 - select all categories with less than 3 items
    @Test
    public void selectItems() {

        EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
                EntityManagerFactory.class).get();

        EntityManager em = entityManagerFactory.createEntityManager();
        List<Category> categories = null;


        Assert.assertEquals(3, categories.size());
        for (Category category : categories) {
            Assert.assertTrue(3 > category.getItems().size());
        }

        em.close();

    }

    // TODO 4 - select categories containing an item sold by John (firstname)
    @Test
    public void selectJohnsCategories() {

        EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
                EntityManagerFactory.class).get();

        EntityManager em = entityManagerFactory.createEntityManager();
        List<Category> categories = null;

        em.close();

        Assert.assertEquals(2, categories.size());
        for (Category category : categories) {
            Assert.assertTrue("Name " + category.getName() + " is not in the list", Arrays.asList("Trecking Bikes", "City Bikes").contains(category.getName()));
        }
    }

    /*
    TODO 5 - Create a report showing the users name and the number of items he is selling and the
    number of categories having items of this user.
     */
    @Test
    public void categoryReport() {

        EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
                EntityManagerFactory.class).get();

        EntityManager em = entityManagerFactory.createEntityManager();
        List<User> result = em.createQuery("select u from User u", User.class).getResultList();

        List<UserReport> userReports = new ArrayList<UserReport>();
        for (User user : result) {
            int items = user.getItemsForSale().size();
            int categories = 0;
            for (Item item : user.getItemsForSale()) {
                categories += item.getCategories().size();
            }

            UserReport userReport = new UserReport(user.getFirstname(), items, categories);
            userReports.add(userReport);
        }

        for (UserReport report : userReports) {
            System.out.println(report);
        }
        em.close();
    }

    // TODO 7 - select users which name starts with jo
    @Test
    public void selectUserWithCriteria() {

        EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
                EntityManagerFactory.class).get();

        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder builder = em.getCriteriaBuilder();
        CriteriaQuery<User> query = builder.createQuery(User.class);


        List<User> users = null;
        em.close();

        Assert.assertEquals(3, users.size());
        for (User user : users) {
            Assert.assertTrue(user.getUsername().startsWith("jo"));
        }

    }

    // TODO 8 - select categories containing an item sold by John (firstname)
    @Test
    public void selectJohnsCategoriesWithCriteria() {

        EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
                EntityManagerFactory.class).get();

        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder builder = em.getCriteriaBuilder();
        CriteriaQuery<Category> query = builder.createQuery(Category.class);
        Root<Category> root = query.from(Category.class);
        Join<Item, User> sellerJoin = root.join(Category_.items).join(Item_.seller);
        query.where(builder.equal(sellerJoin.get(User_.firstname), "John"));
        query.distinct(true);
        List<Category> categories = em.createQuery(query).getResultList();

        em.close();

        Assert.assertEquals(2, categories.size());
        for (Category category : categories) {
            System.out.println(category);
            Assert.assertTrue("Name " + category.getName() + " is not in the list", Arrays.asList("Trecking Bikes", "City Bikes").contains(category.getName()));
        }

    }


}
