package auction.service;

import auction.dao.CategoryDAO;
import auction.dao.ItemDAO;
import auction.model.Bid;
import auction.model.Category;
import auction.model.Item;
import com.jboss.transactions.Transactional;

import javax.inject.Inject;

/**
 * @author Sebastian Hennebrueder
 */
public class AuctionServiceImpl implements AuctionService {
	private ItemDAO itemDAO;
	private CategoryDAO categoryDAO;


	@Inject
	public AuctionServiceImpl(ItemDAO itemDAO, CategoryDAO categoryDAO) {
		this.itemDAO = itemDAO;
		this.categoryDAO = categoryDAO;
	}


	@Override
	@Transactional
	public void create(Item item) {
		itemDAO.persistent(item);
	}

	@Override @Transactional
	public void placeBid(Bid bid) {
		// just make sure that bid is added to collection as well
		bid.getItem().addBid(bid);
		// cascading will save the bid
		itemDAO.merge(bid.getItem());
	}

	@Override @Transactional
	public void create(Category category) {
		categoryDAO.persistent(category);
	}
}
