package auction.service;

import auction.model.*;
import auction.test.DatabaseSetup;
import org.jboss.weld.environment.se.Weld;
import org.jboss.weld.environment.se.WeldContainer;
import org.jboss.weld.environment.se.events.ContainerInitialized;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.Root;
import java.util.*;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

/**
 * @author Sebastian Hennebrueder
 */
public class MappingTest {

    final Logger logger = LoggerFactory.getLogger(MappingTest.class);

    private static WeldContainer weldContainer;

    private static Date today;
    private static Date nextWeek;

    @BeforeClass
    public static void setup() {
        final Calendar calendar = Calendar.getInstance();
        today = calendar.getTime();
        calendar.add(Calendar.DATE, 7);
        nextWeek = calendar.getTime();

        // start the container first to let the Hibernate generate the tables
        weldContainer = new Weld().initialize();
        weldContainer.event().select(ContainerInitialized.class).fire(new ContainerInitialized());


        DatabaseSetup setup = new DatabaseSetup();
        setup.setDataFile("auction/service/basedata.xml").setup();

    }


    @Test
    public void displayUserRepeatedly() {

        EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
                EntityManagerFactory.class).get();


        for (int i = 0; i < 3; i++) {
            System.out.println("-------- Round " + (1 + i) + " ------------ ");
            EntityManager em = entityManagerFactory.createEntityManager();
            User john = em.find(User.class, 1L);
            System.out.println(john);
            for (Item item : john.getItemsForSale()) {
                System.out.println("  " + item);
                for (Category c : item.getCategories()) {
                    System.out.println("    Category: " + c.getName());
                }
            }
            em.close();
        }

    }

    @Test
    public void displayUser() {
        EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
                EntityManagerFactory.class).get();


        EntityManager em = entityManagerFactory.createEntityManager();
        List<User> users = em.createQuery("select u from User u", User.class).getResultList();
        for (User user : users) {
            System.out.println(user.getFirstname());
            for (Item item : user.getItemsForSale()) {
                System.out.println("  " + item);
                for (Category c : item.getCategories()) {
                    System.out.println("    Category: " + c.getName());
                }
            }

        }
        em.close();


    }

    @Test
    public void singleQuery() {

        EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
                EntityManagerFactory.class).get();

        EntityManager em = entityManagerFactory.createEntityManager();
    // TODO 3 - Select John, the items he is selling and the item's categories in a single query
        List<User> users = null;

        em.close();
        for (User user : users) {
            System.out.println(user.getFirstname());
            for (Item item : user.getItemsForSale()) {
                System.out.println("  " + item);
                for (Category c : item.getCategories()) {
                    System.out.println("    Category: " + c.getName());
                }
            }

        }
    }


}
